'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function () {
  var debug = arguments.length <= 0 || arguments[0] === undefined ? _debug2.default : arguments[0];

  return function (chai, utils) {
    var Assertion = chai.Assertion;
    var flag = utils.flag;
    var inspect = utils.inspect;

    function wrapAssertion(assertion, _super) {
      return function (arg1, arg2) {
        var wrapper = (0, _wrap2.default)(flag(this, 'object'));

        if (!wrapper) {
          return _super.apply(this, arguments);
        }

        assertion.call(this, {
          markup: function markup() {
            return debug(wrapper);
          },
          sig: inspect(wrapper),
          wrapper: wrapper,
          arg1: arg1,
          arg2: arg2,
          flag: flag,
          inspect: inspect
        });
      };
    }

    function overwriteMethod(assertion, name) {
      name = name || assertion.name;

      Assertion.overwriteMethod(name, function (_super) {
        return wrapAssertion(assertion, _super);
      });
    }

    function overwriteProperty(assertion, name) {
      name = name || assertion.name;

      Assertion.overwriteProperty(name, function (_super) {
        return wrapAssertion(assertion, _super);
      });
    }

    function overwriteChainableMethod(assertion, name) {
      name = name || assertion.name;

      Assertion.overwriteChainableMethod(name, function (_super) {
        return wrapAssertion(assertion, _super);
      }, function (_super) {
        return function () {
          _super.call(this);
        };
      });
    }

    overwriteMethod((0, _generic2.default)('attr', 'attribute'), 'attr');
    overwriteMethod((0, _generic2.default)('data', 'data attribute'), 'data');
    overwriteMethod((0, _generic2.default)('style', 'CSS style property'), 'style');
    overwriteMethod((0, _generic2.default)('state', 'state'), 'state');
    overwriteMethod((0, _generic2.default)('prop', 'prop'), 'prop');

    overwriteMethod(_checked2.default);
    overwriteMethod(_className2.default);
    overwriteMethod(_disabled2.default);
    overwriteMethod(_id2.default);
    overwriteMethod(_selected2.default);
    overwriteMethod(_value2.default);
    overwriteMethod(_match2.default);
    overwriteMethod(_descendants2.default);
    overwriteMethod(_ref2.default);
    overwriteMethod(_html2.default);
    overwriteMethod(_tagName2.default);
    overwriteMethod(_text2.default);

    overwriteProperty(_empty2.default);
    overwriteMethod(_empty2.default, 'blank');

    overwriteProperty(_exist2.default);
    overwriteMethod(_exist2.default, 'present');

    overwriteChainableMethod(_contain2.default);
  };
};

var _wrap = require('./wrap');

var _wrap2 = _interopRequireDefault(_wrap);

var _debug = require('./debug');

var _debug2 = _interopRequireDefault(_debug);

var _checked = require('./assertions/checked');

var _checked2 = _interopRequireDefault(_checked);

var _className = require('./assertions/className');

var _className2 = _interopRequireDefault(_className);

var _contain = require('./assertions/contain');

var _contain2 = _interopRequireDefault(_contain);

var _descendants = require('./assertions/descendants');

var _descendants2 = _interopRequireDefault(_descendants);

var _disabled = require('./assertions/disabled');

var _disabled2 = _interopRequireDefault(_disabled);

var _empty = require('./assertions/empty');

var _empty2 = _interopRequireDefault(_empty);

var _exist = require('./assertions/exist');

var _exist2 = _interopRequireDefault(_exist);

var _generic = require('./assertions/generic');

var _generic2 = _interopRequireDefault(_generic);

var _html = require('./assertions/html');

var _html2 = _interopRequireDefault(_html);

var _id = require('./assertions/id');

var _id2 = _interopRequireDefault(_id);

var _match = require('./assertions/match');

var _match2 = _interopRequireDefault(_match);

var _ref = require('./assertions/ref');

var _ref2 = _interopRequireDefault(_ref);

var _selected = require('./assertions/selected');

var _selected2 = _interopRequireDefault(_selected);

var _tagName = require('./assertions/tagName');

var _tagName2 = _interopRequireDefault(_tagName);

var _text = require('./assertions/text');

var _text2 = _interopRequireDefault(_text);

var _value = require('./assertions/value');

var _value2 = _interopRequireDefault(_value);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }